=== modified file 'NEWS'
--- NEWS	2005-09-02 02:06:23 +0000
+++ NEWS	2005-09-12 01:29:49 +0000
@@ -1,3 +1,32 @@
+development head
+
+  IMPROVEMENTS:
+
+    * Adding a file whose parent directory is not versioned will
+      implicitly add the parent, and so on up to the root. This means
+      you should never need to explictly add a directory, they'll just
+      get added when you add a file in the directory.  Contributed by
+      Michael Ellerman.
+
+
+  INTERNALS:
+
+    * Refactor xml packing/unpacking.
+
+
+  BUG FIXES: 
+
+    * Fixed 'bzr mv' by Ollie Rutherfurd.
+
+    * Fixed strange error when trying to access a nonexistent http
+      branch.
+
+
+  PORTABILITY:
+
+    * Various Windows fixes from Ollie Rutherfurd.
+
+
 bzr-0.0.7 2005-09-02
 
   NEW FEATURES:

=== modified file 'bzrlib/add.py'
--- bzrlib/add.py	2005-09-07 10:47:36 +0000
+++ bzrlib/add.py	2005-09-12 12:28:19 +0000
@@ -18,6 +18,7 @@
 from bzrlib.errors import NotBranchError
 from bzrlib.branch import Branch
 from bzrlib.osutils import quotefn
+from os.path import dirname
 
 def glob_expand_for_win32(file_list):
     import glob
@@ -78,11 +79,8 @@
     Returns the number of files added.
     """
     import os
-    import sys
-    from bzrlib.osutils import quotefn
     from bzrlib.errors import BadFileKindError, ForbiddenFileError
     import bzrlib.branch
-    import bzrlib.osutils
 
     assert isinstance(recurse, bool)
 
@@ -129,10 +127,7 @@
         elif sub_tree:
             mutter("%r is a bzr tree" %f)
         else:
-            entry = inv.add_path(rf, kind=kind)
-            mutter("added %r kind %r file_id={%s}" % (rf, kind, entry.file_id))
-            count += 1 
-            reporter(rf, kind, entry)
+            count += __add_one(branch, inv, rf, kind, reporter)
 
         if kind == 'directory' and recurse and not sub_tree:
             for subf in os.listdir(af):
@@ -152,3 +147,21 @@
         branch._write_inventory(inv)
 
     return count
+
+def __add_one(branch, inv, path, kind, reporter):
+    """Add a file or directory, automatically add unversioned parents."""
+
+    # Nothing to do if path is already versioned.
+    # This is safe from infinite recursion because the branch root is
+    # always versioned.
+    if inv.path2id(path) != None:
+        return 0
+
+    # add parent
+    count = __add_one(branch, inv, dirname(path), 'directory', reporter)
+
+    entry = inv.add_path(path, kind=kind)
+    mutter("added %r kind %r file_id={%s}" % (path, kind, entry.file_id))
+    reporter(path, kind, entry)
+
+    return count + 1

=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2005-09-07 10:47:36 +0000
+++ bzrlib/branch.py	2005-09-12 12:49:28 +0000
@@ -28,9 +28,9 @@
 import bzrlib.errors
 from bzrlib.textui import show_status
 from bzrlib.revision import Revision
-from bzrlib.xml import unpack_xml
 from bzrlib.delta import compare_trees
 from bzrlib.tree import EmptyTree, RevisionTree
+import bzrlib.xml
 import bzrlib.ui
 
 
@@ -301,7 +301,6 @@
 
     def _make_control(self):
         from bzrlib.inventory import Inventory
-        from bzrlib.xml import pack_xml
         
         os.mkdir(self.controlfilename([]))
         self.controlfile('README', 'w').write(
@@ -320,7 +319,9 @@
         # if we want per-tree root ids then this is the place to set
         # them; they're not needed for now and so ommitted for
         # simplicity.
-        pack_xml(Inventory(), self.controlfile('inventory','w'))
+        f = self.controlfile('inventory','w')
+        bzrlib.xml.serializer_v4.write_inventory(Inventory(), f)
+
 
     def _check_format(self):
         """Check this branch format is supported.
@@ -334,7 +335,7 @@
         # on Windows from Linux and so on.  I think it might be better
         # to always make all internal files in unix format.
         fmt = self.controlfile('branch-format', 'r').read()
-        fmt.replace('\r\n', '')
+        fmt = fmt.replace('\r\n', '\n')
         if fmt != BZR_BRANCH_FORMAT:
             raise BzrError('sorry, branch format %r not supported' % fmt,
                            ['use a different bzr version',
@@ -360,18 +361,12 @@
     def read_working_inventory(self):
         """Read the working inventory."""
         from bzrlib.inventory import Inventory
-        from bzrlib.xml import unpack_xml
-        from time import time
-        before = time()
         self.lock_read()
         try:
             # ElementTree does its own conversion from UTF-8, so open in
             # binary.
-            inv = unpack_xml(Inventory,
-                             self.controlfile('inventory', 'rb'))
-            mutter("loaded inventory of %d items in %f"
-                   % (len(inv), time() - before))
-            return inv
+            f = self.controlfile('inventory', 'rb')
+            return bzrlib.xml.serializer_v4.read_inventory(f)
         finally:
             self.unlock()
             
@@ -383,13 +378,12 @@
         will be committed to the next revision.
         """
         from bzrlib.atomicfile import AtomicFile
-        from bzrlib.xml import pack_xml
         
         self.lock_write()
         try:
             f = AtomicFile(self.controlfilename('inventory'), 'wb')
             try:
-                pack_xml(inv, f)
+                bzrlib.xml.serializer_v4.write_inventory(inv, f)
                 f.commit()
             finally:
                 f.close()
@@ -577,7 +571,7 @@
         finally:
             f.close()
 
-    def get_revision_xml(self, revision_id):
+    def get_revision_xml_file(self, revision_id):
         """Return XML file object for revision object."""
         if not revision_id or not isinstance(revision_id, basestring):
             raise InvalidRevisionId(revision_id)
@@ -591,16 +585,15 @@
         finally:
             self.unlock()
 
+    #deprecated
+    get_revision_xml = get_revision_xml_file
+
     def get_revision(self, revision_id):
-        """Return the Revision object for a named revision. 
-        
-        This operates on the storage level - it is agnostic as to
-        logical presence or absence from the branch.
-        """
-        xml_file = self.get_revision_xml(revision_id)
+        """Return the Revision object for a named revision"""
+        xml_file = self.get_revision_xml_file(revision_id)
 
         try:
-            r = unpack_xml(Revision, xml_file)
+            r = bzrlib.xml.serializer_v4.read_revision(xml_file)
         except SyntaxError, e:
             raise bzrlib.errors.BzrError('failed to unpack revision_xml',
                                          [revision_id,
@@ -647,13 +640,14 @@
                parameter which can be either an integer revno or a
                string hash."""
         from bzrlib.inventory import Inventory
-        from bzrlib.xml import unpack_xml
-
-        return unpack_xml(Inventory, self.get_inventory_xml(inventory_id))
+        f = self.get_inventory_xml_file(inventory_id)
+        return bzrlib.xml.serializer_v4.read_inventory(f)
 
     def get_inventory_xml(self, inventory_id):
         """Get inventory XML as a file object."""
         return self.inventory_store[inventory_id]
+
+    get_inventory_xml_file = get_inventory_xml
             
     def get_inventory_sha1(self, inventory_id):
         """Return the sha1 hash of the inventory entry
@@ -795,11 +789,22 @@
         """Pull in all new revisions from other branch.
         """
         from bzrlib.fetch import greedy_fetch
+        from bzrlib.revision import get_intervening_revisions
 
         pb = bzrlib.ui.ui_factory.progress_bar()
         pb.update('comparing histories')
 
-        revision_ids = self.missing_revisions(other, stop_revision)
+        try:
+            revision_ids = self.missing_revisions(other, stop_revision)
+        except DivergedBranches, e:
+            try:
+                if stop_revision is None:
+                    end_revision = other.last_patch()
+                revision_ids = get_intervening_revisions(self.last_patch(), 
+                                                         end_revision, other)
+                assert self.last_patch() not in revision_ids
+            except bzrlib.errors.NotAncestor:
+                raise e
 
         if len(revision_ids) > 0:
             count = greedy_fetch(self, other, revision_ids[-1], pb)[0]

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2005-09-05 05:35:25 +0000
+++ bzrlib/builtins.py	2005-09-12 01:29:49 +0000
@@ -62,11 +62,10 @@
     directory is shown.  Otherwise, only the status of the specified
     files or directories is reported.  If a directory is given, status
     is reported for everything inside that directory.
+    """
 
-    If a revision is specified, the changes since that revision are shown.
-    """
     takes_args = ['file*']
-    takes_options = ['all', 'show-ids', 'revision']
+    takes_options = ['all', 'show-ids']
     aliases = ['st', 'stat']
     
     def run(self, all=False, show_ids=False, file_list=None):
@@ -92,8 +91,8 @@
     takes_args = ['revision_id']
     
     def run(self, revision_id):
-        from bzrlib.xml import pack_xml
-        pack_xml(find_branch('.').get_revision(revision_id), sys.stdout)
+        b = find_branch('.')
+        sys.stdout.write(b.get_revision_xml_file(revision_id).read())
 
 
 class cmd_revno(Command):
@@ -103,6 +102,7 @@
     def run(self):
         print find_branch('.').revno()
 
+
 class cmd_revision_info(Command):
     """Show revision number and revision id for a given revision identifier.
     """
@@ -144,8 +144,10 @@
     Therefore simply saying 'bzr add' will version all files that
     are currently unknown.
 
-    TODO: Perhaps adding a file whose directly is not versioned should
-    recursively add that parent, rather than giving an error?
+    Adding a file whose parent directory is not versioned will
+    implicitly add the parent, and so on up to the root. This means
+    you should never need to explictly add a directory, they'll just
+    get added when you add a file in the directory.
     """
     takes_args = ['file*']
     takes_options = ['verbose', 'no-recurse']
@@ -535,7 +537,7 @@
     examples:
         bzr diff
         bzr diff -r1
-        bzr diff -r1:2
+        bzr diff -r1..2
     """
     
     takes_args = ['file*']

=== modified file 'bzrlib/commit.py'
--- bzrlib/commit.py	2005-09-07 10:47:36 +0000
+++ bzrlib/commit.py	2005-09-12 12:28:19 +0000
@@ -16,8 +16,6 @@
 
 
 
-# FIXME: "bzr commit doc/format" commits doc/format.txt!
-
 def commit(branch, message,
            timestamp=None,
            timezone=None,
@@ -66,7 +64,7 @@
     from bzrlib.errors import BzrError, PointlessCommit
     from bzrlib.revision import Revision, RevisionReference
     from bzrlib.trace import mutter, note
-    from bzrlib.xml import pack_xml
+    from bzrlib.xml import serializer_v4
 
     branch.lock_write()
 
@@ -123,7 +121,8 @@
         inv_id = rev_id
 
         inv_tmp = tempfile.TemporaryFile()
-        pack_xml(new_inv, inv_tmp)
+        
+        serializer_v4.write_inventory(new_inv, inv_tmp)
         inv_tmp.seek(0)
         branch.inventory_store.add(inv_tmp, inv_id)
         mutter('new inventory_id is {%s}' % inv_id)
@@ -162,7 +161,7 @@
             rev.parents.append(RevisionReference(merge_rev))            
 
         rev_tmp = tempfile.TemporaryFile()
-        pack_xml(rev, rev_tmp)
+        serializer_v4.write_revision(rev, rev_tmp)
         rev_tmp.seek(0)
         branch.revision_store.add(rev_tmp, rev_id)
         mutter("new revision_id is {%s}" % rev_id)

=== added file 'bzrlib/graph.py'
--- bzrlib/graph.py	1970-01-01 00:00:00 +0000
+++ bzrlib/graph.py	2005-09-12 02:53:07 +0000
@@ -0,0 +1,93 @@
+# (C) 2005 Canonical
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+def max_distance(node, ancestors, distances, root_descendants):
+    """Calculate the max distance to an ancestor.  
+    Return None if not all possible ancestors have known distances"""
+    best = None
+    if node in distances:
+        best = distances[node]
+    for ancestor in ancestors[node]:
+        # skip ancestors we will never traverse:
+        if root_descendants is not None and ancestor not in root_descendants:
+            continue
+        # An ancestor which is not listed in ancestors will never be in
+        # distances, so we pretend it never existed.
+        if ancestor not in ancestors:
+            continue
+        if ancestor not in distances:
+            return None
+        if best is None or distances[ancestor] > best:
+            best = distances[ancestor] + 1
+    return best
+
+    
+def node_distances(graph, ancestors, start, root_descendants=None):
+    """Produce a list of nodes, sorted by distance from a start node.
+    This is an algorithm devised by Aaron Bentley, because applying Dijkstra
+    backwards seemed too complicated.
+
+    For each node, we walk its descendants.  If all the descendant's ancestors
+    have a max-distance-to-start, (excluding ones that can never reach start),
+    we calculate their max-distance-to-start, and schedule their descendants.
+
+    So when a node's last parent acquires a distance, it will acquire a
+    distance on the next iteration.
+
+    Once we know the max distances for all nodes, we can return a list sorted
+    by distance, farthest first.
+    """
+    distances = {start: 0}
+    lines = set([start])
+    while len(lines) > 0:
+        new_lines = set()
+        for line in lines:
+            line_descendants = graph[line]
+            assert line not in line_descendants, "%s refers to itself" % line
+            for descendant in line_descendants:
+                distance = max_distance(descendant, ancestors, distances,
+                                        root_descendants)
+                if distance is None:
+                    continue
+                distances[descendant] = distance
+                new_lines.add(descendant)
+        lines = new_lines
+    return distances
+
+def farthest_nodes(graph, ancestors, start):
+    def by_distance(n):
+        return distances[n],n
+
+    distances = node_distances(graph, ancestors, start)
+    node_list = distances.keys()
+    node_list.sort(key=by_distance, reverse=True)
+    return node_list
+
+def all_descendants(descendants, start):
+    result = set()
+    lines = set([start])
+    while len(lines) > 0:
+        new_lines = set()
+        for line in lines:
+            if line not in descendants:
+                continue
+            for descendant in descendants[line]:
+                if descendant in result:
+                    continue
+                result.add(descendant)
+                new_lines.add(descendant)
+        lines = new_lines
+    return result

=== modified file 'bzrlib/inventory.py'
--- bzrlib/inventory.py	2005-09-07 10:47:36 +0000
+++ bzrlib/inventory.py	2005-09-12 12:49:28 +0000
@@ -100,7 +100,9 @@
     # directories, etc etc.
 
     __slots__ = ['text_sha1', 'text_size', 'file_id', 'name', 'kind',
-                 'text_id', 'parent_id', 'children', 'symlink_target']
+                 'text_id', 'parent_id', 'children',
+                 'text_version', 'entry_version', 'symlink_target']
+
 
     def __init__(self, file_id, name, kind, parent_id, text_id=None):
         """Create an InventoryEntry
@@ -117,12 +119,14 @@
         Traceback (most recent call last):
         BzrCheckError: InventoryEntry name 'src/hello.c' is invalid
         """
+        assert isinstance(name, basestring), name
         if '/' in name or '\\' in name:
             raise BzrCheckError('InventoryEntry name %r is invalid' % name)
         
+        self.text_version = None
+        self.entry_version = None
         self.text_sha1 = None
         self.text_size = None
-    
         self.file_id = file_id
         self.name = name
         self.kind = kind
@@ -170,62 +174,6 @@
                    self.kind,
                    self.parent_id))
 
-    
-    def to_element(self):
-        """Convert to XML element"""
-        from bzrlib.xml import Element
-        
-        e = Element('entry')
-
-        e.set('name', self.name)
-        e.set('file_id', self.file_id)
-        e.set('kind', self.kind)
-
-        if self.text_size != None:
-            e.set('text_size', '%d' % self.text_size)
-            
-        for f in ['text_id', 'text_sha1', 'symlink_target']:
-            v = getattr(self, f)
-            if v != None:
-                e.set(f, v)
-
-        # to be conservative, we don't externalize the root pointers
-        # for now, leaving them as null in the xml form.  in a future
-        # version it will be implied by nested elements.
-        if self.parent_id != ROOT_ID:
-            assert isinstance(self.parent_id, basestring)
-            e.set('parent_id', self.parent_id)
-
-        e.tail = '\n'
-            
-        return e
-
-
-    def from_element(cls, elt):
-        assert elt.tag == 'entry'
-
-        ## original format inventories don't have a parent_id for
-        ## nodes in the root directory, but it's cleaner to use one
-        ## internally.
-        parent_id = elt.get('parent_id')
-        if parent_id == None:
-            parent_id = ROOT_ID
-
-        self = cls(elt.get('file_id'), elt.get('name'), elt.get('kind'), parent_id)
-        self.text_id = elt.get('text_id')
-        self.text_sha1 = elt.get('text_sha1')
-        self.symlink_target = elt.get('symlink_target')
-        
-        ## mutter("read inventoryentry: %r" % (elt.attrib))
-
-        v = elt.get('text_size')
-        self.text_size = v and int(v)
-
-        return self
-            
-
-    from_element = classmethod(from_element)
-
     def __eq__(self, other):
         if not isinstance(other, InventoryEntry):
             return NotImplemented
@@ -237,8 +185,9 @@
                and (self.text_size == other.text_size) \
                and (self.text_id == other.text_id) \
                and (self.parent_id == other.parent_id) \
-               and (self.kind == other.kind)
-
+               and (self.kind == other.kind) \
+               and (self.text_version == other.text_version) \
+               and (self.entry_version == other.entry_version)
 
     def __ne__(self, other):
         return not (self == other)
@@ -427,7 +376,10 @@
         """Add entry to inventory.
 
         To add  a file to a branch ready to be committed, use Branch.add,
-        which calls this."""
+        which calls this.
+
+        Returns the new entry object.
+        """
         if entry.file_id in self._byid:
             raise BzrError("inventory already contains entry with id {%s}" % entry.file_id)
 
@@ -451,7 +403,9 @@
     def add_path(self, relpath, kind, file_id=None):
         """Add entry from a path.
 
-        The immediate parent must already be versioned"""
+        The immediate parent must already be versioned.
+
+        Returns the new entry object."""
         from bzrlib.branch import gen_file_id
         
         parts = bzrlib.osutils.splitpath(relpath)
@@ -498,44 +452,6 @@
         del self[ie.parent_id].children[ie.name]
 
 
-    def to_element(self):
-        """Convert to XML Element"""
-        from bzrlib.xml import Element
-        
-        e = Element('inventory')
-        e.text = '\n'
-        if self.root.file_id not in (None, ROOT_ID):
-            e.set('file_id', self.root.file_id)
-        for path, ie in self.iter_entries():
-            e.append(ie.to_element())
-        return e
-    
-
-    def from_element(cls, elt):
-        """Construct from XML Element
-        
-        >>> inv = Inventory()
-        >>> inv.add(InventoryEntry('foo.c-123981239', 'foo.c', 'file', ROOT_ID))
-        InventoryEntry('foo.c-123981239', 'foo.c', kind='file', parent_id='TREE_ROOT')
-        >>> elt = inv.to_element()
-        >>> inv2 = Inventory.from_element(elt)
-        >>> inv2 == inv
-        True
-        """
-        # XXXX: doctest doesn't run this properly under python2.3
-        assert elt.tag == 'inventory'
-        root_id = elt.get('file_id') or ROOT_ID
-        o = cls(root_id)
-        for e in elt:
-            ie = InventoryEntry.from_element(e)
-            if ie.parent_id == ROOT_ID:
-                ie.parent_id = root_id
-            o.add(ie)
-        return o
-        
-    from_element = classmethod(from_element)
-
-
     def __eq__(self, other):
         """Compare two sets by comparing their contents.
 
@@ -570,7 +486,6 @@
         raise ValueError('not hashable')
 
 
-
     def get_idpath(self, file_id):
         """Return a list of file_ids for the path to an entry.
 

=== modified file 'bzrlib/merge.py'
--- bzrlib/merge.py	2005-09-05 02:01:31 +0000
+++ bzrlib/merge.py	2005-09-07 01:34:07 +0000
@@ -15,7 +15,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 
-import os.path
+import os
 import tempfile
 import shutil
 import errno
@@ -357,7 +357,7 @@
             if path == '.':
                 path = ''
             else:
-                assert path.startswith('./'), "path is %s" % path
+                assert path.startswith('.' + os.sep), "path is %s" % path
             path = path[2:]
         adjust_ids.append((path, id))
     if len(adjust_ids) > 0:

=== modified file 'bzrlib/msgeditor.py'
--- bzrlib/msgeditor.py	2005-09-01 11:27:20 +0000
+++ bzrlib/msgeditor.py	2005-09-07 01:34:07 +0000
@@ -20,6 +20,7 @@
 """Commit message editor support."""
 
 import os
+from subprocess import call
 from bzrlib.errors import BzrError
 
 def _get_editor():
@@ -30,12 +31,12 @@
     if e is not None:
         yield e
         
-    if os.name == "windows":
-        yield "notepad.exe"
-    elif os.name == "posix":
-        try:
-            yield os.environ["EDITOR"]
-        except KeyError:
+    try:
+        yield os.environ["EDITOR"]
+    except KeyError:
+        if os.name == "nt":
+            yield "notepad.exe"
+        elif os.name == "posix":
             yield "/usr/bin/vi"
 
 
@@ -43,8 +44,7 @@
     """Try to execute an editor to edit the commit message."""
     for e in _get_editor():
         edargs = e.split(' ')
-        x = os.spawnvp(os.P_WAIT, edargs[0],
-                       edargs + [filename])
+        x = call(edargs + [filename])
         if x == 0:
             return True
         elif x == 127:

=== modified file 'bzrlib/remotebranch.py'
--- bzrlib/remotebranch.py	2005-09-06 15:39:59 +0000
+++ bzrlib/remotebranch.py	2005-09-12 12:28:19 +0000
@@ -26,14 +26,14 @@
 
 import gzip
 from cStringIO import StringIO
+import os
 import urllib2
-
-from errors import BzrError, BzrCheckError
-from branch import Branch, BZR_BRANCH_FORMAT
-from trace import mutter
-
-# velocitynet.com.au transparently proxies connections and thereby
-# breaks keep-alive -- sucks!
+import urlparse
+
+from bzrlib.errors import BzrError, BzrCheckError
+from bzrlib.branch import Branch, BZR_BRANCH_FORMAT
+from bzrlib.trace import mutter
+from bzrlib.xml import serializer_v4
 
 
 ENABLE_URLGRABBER = False
@@ -84,8 +84,8 @@
     orig_url = url
     while True:
         try:
-            ff = get_url(url + '/.bzr/branch-format')
-
+            fmt_url = url + '/.bzr/branch-format'
+            ff = get_url(fmt_url)
             fmt = ff.read()
             ff.close()
 
@@ -98,12 +98,17 @@
         except urllib2.URLError:
             pass
 
-        try:
-            idx = url.rindex('/')
-        except ValueError:
-            raise BzrError('no branch root found for URL %s' % orig_url)
-
-        url = url[:idx]        
+        scheme, host, path = list(urlparse.urlparse(url))[:3]
+        # discard params, query, fragment
+        
+        # strip off one component of the path component
+        idx = path.rfind('/')
+        if idx == -1 or path == '/':
+            raise BzrError('no branch root found for URL %s'
+                           ' or enclosing directories'
+                           % orig_url)
+        path = path[:idx]
+        url = urlparse.urlunparse((scheme, host, path, '', '', ''))
         
 
 class RemoteBranch(Branch):
@@ -155,13 +160,11 @@
 
 
     def get_revision(self, revision_id):
-        from bzrlib.revision import Revision
-        from bzrlib.xml import unpack_xml
         try:
             revf = self.revision_store[revision_id]
         except KeyError:
             raise NoSuchRevision(self, revision_id)
-        r = unpack_xml(Revision, revf)
+        r = serializer_v4.read_revision(revf)
         if r.revision_id != revision_id:
             raise BzrCheckError('revision stored as {%s} actually contains {%s}'
                                 % (revision_id, r.revision_id))

=== modified file 'bzrlib/revision.py'
--- bzrlib/revision.py	2005-09-07 16:31:54 +0000
+++ bzrlib/revision.py	2005-09-12 12:28:19 +0000
@@ -16,7 +16,7 @@
 
 
 import bzrlib.errors
-
+from bzrlib.graph import farthest_nodes, node_distances, all_descendants
 
 class RevisionReference(object):
     """
@@ -94,89 +94,20 @@
 
     def __repr__(self):
         return "<Revision id %s>" % self.revision_id
-        
-    def to_element(self):
-        from bzrlib.xml import Element, SubElement
-        
-        root = Element('revision',
-                       committer = self.committer,
-                       timestamp = '%.9f' % self.timestamp,
-                       revision_id = self.revision_id,
-                       inventory_id = self.inventory_id,
-                       inventory_sha1 = self.inventory_sha1,
-                       )
-        if self.timezone:
-            root.set('timezone', str(self.timezone))
-        root.text = '\n'
-        
-        msg = SubElement(root, 'message')
-        msg.text = self.message
-        msg.tail = '\n'
-
-        if self.parents:
-            pelts = SubElement(root, 'parents')
-            pelts.tail = pelts.text = '\n'
-            for rr in self.parents:
-                assert isinstance(rr, RevisionReference)
-                p = SubElement(pelts, 'revision_ref')
-                p.tail = '\n'
-                assert rr.revision_id
-                p.set('revision_id', rr.revision_id)
-                if rr.revision_sha1:
-                    p.set('revision_sha1', rr.revision_sha1)
-
-        return root
-
-
-    def from_element(cls, elt):
-        return unpack_revision(elt)
-
-    from_element = classmethod(from_element)
-
-
-
-def unpack_revision(elt):
-    """Convert XML element into Revision object."""
-    # <changeset> is deprecated...
-    if elt.tag not in ('revision', 'changeset'):
-        raise bzrlib.errors.BzrError("unexpected tag in revision file: %r" % elt)
-
-    rev = Revision(committer = elt.get('committer'),
-                   timestamp = float(elt.get('timestamp')),
-                   revision_id = elt.get('revision_id'),
-                   inventory_id = elt.get('inventory_id'),
-                   inventory_sha1 = elt.get('inventory_sha1')
-                   )
-
-    precursor = elt.get('precursor')
-    precursor_sha1 = elt.get('precursor_sha1')
-
-    pelts = elt.find('parents')
-
-    if pelts:
-        for p in pelts:
-            assert p.tag == 'revision_ref', \
-                   "bad parent node tag %r" % p.tag
-            rev_ref = RevisionReference(p.get('revision_id'),
-                                        p.get('revision_sha1'))
-            rev.parents.append(rev_ref)
-
-        if precursor:
-            # must be consistent
-            prec_parent = rev.parents[0].revision_id
-            assert prec_parent == precursor
-    elif precursor:
-        # revisions written prior to 0.0.5 have a single precursor
-        # give as an attribute
-        rev_ref = RevisionReference(precursor, precursor_sha1)
-        rev.parents.append(rev_ref)
-
-    v = elt.get('timezone')
-    rev.timezone = v and int(v)
-
-    rev.message = elt.findtext('message') # text of <message>
-    return rev
-
+
+    def __eq__(self, other):
+        if not isinstance(other, Revision):
+            return False
+        return (self.inventory_id == other.inventory_id
+                and self.inventory_sha1 == other.inventory_sha1
+                and self.revision_id == other.revision_id
+                and self.timestamp == other.timestamp
+                and self.message == other.message
+                and self.timezone == other.timezone
+                and self.committer == other.committer)
+
+    def __ne__(self, other):
+        return not self.__eq__(other)
 
 
 REVISION_ID_RE = None
@@ -186,7 +117,7 @@
     global REVISION_ID_RE
     if not REVISION_ID_RE:
         import re
-        REVISION_ID_RE = re.compile('[\w.-]+@[\w.-]+--?\d+--?[0-9a-f]+\Z')
+        REVISION_ID_RE = re.compile('[\w:.-]+@[\w%.-]+--?[\w]+--?[0-9a-f]+\Z')
 
     if not REVISION_ID_RE.match(rid):
         raise ValueError("malformed revision-id %r" % rid)
@@ -252,7 +183,7 @@
     return matches
 
 
-def common_ancestor(revision_a, revision_b, revision_source):
+def old_common_ancestor(revision_a, revision_b, revision_source):
     """Find the ancestor common to both revisions that is closest to both.
     """
     from bzrlib.trace import mutter
@@ -283,6 +214,73 @@
         raise bzrlib.errors.AmbiguousBase((a_closest[0], b_closest[0]))
     return a_closest[0]
 
+def revision_graph(revision, revision_source):
+    """Produce a graph of the ancestry of the specified revision.
+    Return root, ancestors map, descendants map
+
+    TODO: Produce graphs with the NULL revision as root, so that we can find
+    a common even when trees are not branches don't represent a single line
+    of descent.
+    """
+    ancestors = {}
+    descendants = {}
+    lines = [revision]
+    root = None
+    descendants[revision] = {}
+    while len(lines) > 0:
+        new_lines = set()
+        for line in lines:
+            try:
+                rev = revision_source.get_revision(line)
+                parents = [p.revision_id for p in rev.parents]
+                if len(parents) == 0:
+                    root = line
+            except bzrlib.errors.NoSuchRevision:
+                if line == revision:
+                    raise
+                parents = None
+            if parents is not None:
+                for parent in parents:
+                    if parent not in ancestors:
+                        new_lines.add(parent)
+                    if parent not in descendants:
+                        descendants[parent] = {}
+                    descendants[parent][line] = 1
+            if parents is not None:
+                ancestors[line] = set(parents)
+        lines = new_lines
+    assert root not in descendants[root]
+    assert root not in ancestors[root]
+    return root, ancestors, descendants
+
+def combined_graph(revision_a, revision_b, revision_source):
+    """Produce a combined ancestry graph.
+    Return graph root, ancestors map, descendants map, set of common nodes"""
+    root, ancestors, descendants = revision_graph(revision_a, revision_source)
+    root_b, ancestors_b, descendants_b = revision_graph(revision_b, 
+                                                        revision_source)
+    assert root == root_b
+    common = set()
+    for node, node_anc in ancestors_b.iteritems():
+        if node in ancestors:
+            common.add(node)
+        else:
+            ancestors[node] = set()
+        ancestors[node].update(node_anc)
+    for node, node_dec in descendants_b.iteritems():
+        if node not in descendants:
+            descendants[node] = set()
+        descendants[node].update(node_dec)
+    return root, ancestors, descendants, common
+
+def common_ancestor(revision_a, revision_b, revision_source):
+    root, ancestors, descendants, common = \
+        combined_graph(revision_a, revision_b, revision_source)
+    nodes = farthest_nodes(descendants, ancestors, root)
+    for node in nodes:
+        if node in common:
+            return node
+
 class MultipleRevisionSources(object):
     """Proxy that looks in multiple branches for revisions."""
     def __init__(self, *args):
@@ -307,49 +305,36 @@
     Otherwise, rev_id will be the last entry.  ancestor_id will never appear.
     If ancestor_id is not an ancestor, NotAncestor will be thrown
     """
-    [rev_source.get_revision(r) for r in (ancestor_id, rev_id)]
-    if ancestor_id == rev_id:
-        return []
-    def historical_lines(line):
-        """Return a tuple of historical/non_historical lines, for sorting.
-        The non_historical count is negative, since non_historical lines are
-        a bad thing.
-        """
-        good_count = 0
-        bad_count = 0
-        for revision in line:
-            if revision in revision_history:
-                good_count += 1
-            else:
-                bad_count -= 1
-        return good_count, bad_count
-    active = [[rev_id]]
-    successful_lines = []
-    while len(active) > 0:
-        new_active = []
-        for line in active:
-            parent_ids = [p.revision_id for p in 
-                          rev_source.get_revision(line[-1]).parents]
-            for parent in parent_ids:
-                line_copy = line[:]
-                if parent == ancestor_id:
-                    successful_lines.append(line_copy)
-                else:
-                    line_copy.append(parent)
-                    new_active.append(line_copy)
-        active = new_active
-    if len(successful_lines) == 0:
-        raise bzrlib.errors.NotAncestor(rev_id, ancestor_id)
-    for line in successful_lines:
-        line.reverse()
-    if revision_history is not None:
-        by_historical_lines = []
-        for line in successful_lines:
-            count = historical_lines(line)
-            by_historical_lines.append((count, line))
-        by_historical_lines.sort()
-        if by_historical_lines[-1][0][0] > 0:
-            return by_historical_lines[-1][1]
-    assert len(successful_lines)
-    successful_lines.sort(cmp, len)
-    return successful_lines[-1]
+    root, ancestors, descendants = revision_graph(rev_id, rev_source)
+    if len(descendants) == 0:
+        raise NoSuchRevision(rev_source, rev_id)
+    if ancestor_id not in descendants:
+        rev_source.get_revision(ancestor_id)
+        raise bzrlib.errors.NotAncestor(rev_id, ancestor_id)
+    root_descendants = all_descendants(descendants, ancestor_id)
+    root_descendants.add(ancestor_id)
+    if rev_id not in root_descendants:
+        raise bzrlib.errors.NotAncestor(rev_id, ancestor_id)
+    distances = node_distances(descendants, ancestors, ancestor_id,
+                               root_descendants=root_descendants)
+
+    def best_ancestor(rev_id):
+        best = None
+        for anc_id in ancestors[rev_id]:
+            try:
+                distance = distances[anc_id]
+            except KeyError:
+                continue
+            if revision_history is not None and anc_id in revision_history:
+                return anc_id
+            elif best is None or distance > best[1]:
+                best = (anc_id, distance)
+        return best[0]
+
+    next = rev_id
+    path = []
+    while next != ancestor_id:
+        path.append(next)
+        next = best_ancestor(next)
+    path.reverse()
+    return path

=== modified file 'bzrlib/selftest/__init__.py'
--- bzrlib/selftest/__init__.py	2005-09-05 05:35:25 +0000
+++ bzrlib/selftest/__init__.py	2005-09-12 08:21:24 +0000
@@ -74,6 +74,9 @@
         
         self._log_file_name = name
 
+    def run(self, result):
+        self.apply_redirected(None, None, None,
+                              unittest.TestCase.run, self, result)
         
     def tearDown(self):
         logging.getLogger('').removeHandler(self._log_hdlr)
@@ -138,9 +141,15 @@
         if stdin is None:
             stdin = StringIO("")
         if stdout is None:
-            stdout = self._log_file
+            if hasattr(self, "_log_file"):
+                stdout = self._log_file
+            else:
+                stdout = StringIO()
         if stderr is None:
-            stderr = self._log_file
+            if hasattr(self, "_log_file"):
+                stderr = self._log_file
+            else:
+                stderr = StringIO()
         real_stdin = sys.stdin
         real_stdout = sys.stdout
         real_stderr = sys.stderr
@@ -321,8 +330,10 @@
                    'bzrlib.selftest.test_merge_core',
                    'bzrlib.selftest.test_smart_add',
                    'bzrlib.selftest.testdiff',
+                   'bzrlib.selftest.test_xml',
                    'bzrlib.fetch',
                    'bzrlib.selftest.teststore',
+                   'bzrlib.selftest.testgraph',
                    ]
 
     for m in (bzrlib.store, bzrlib.inventory, bzrlib.branch,

=== modified file 'bzrlib/selftest/blackbox.py'
--- bzrlib/selftest/blackbox.py	2005-09-07 10:47:36 +0000
+++ bzrlib/selftest/blackbox.py	2005-09-12 12:28:19 +0000
@@ -28,6 +28,7 @@
 
 import os;
 import sys
+import os
 
 from bzrlib.selftest import TestCaseInTempDir, BzrTestBase
 from bzrlib.branch import Branch
@@ -123,7 +124,6 @@
         assert file('.bzrignore', 'rb').read() == '*.blah\ngarh\n'
 
     def test_revert(self):
-        import os
         self.runbzr('init')
 
         file('hello', 'wt').write('foo')
@@ -195,7 +195,6 @@
 
     def test_merge(self):
         from bzrlib.branch import Branch
-        import os
         
         os.mkdir('a')
         os.chdir('a')
@@ -217,9 +216,57 @@
         a = Branch('.')
         b = Branch('../b')
         a.get_revision_xml(b.last_patch())
-        print "Pending: %s" % a.pending_merges()
-#        assert a.pending_merges() == [b.last_patch()], "Assertion %s %s" \
-#        % (a.pending_merges(), b.last_patch())
+        self.log('pending merges: %s', a.pending_merges())
+        #        assert a.pending_merges() == [b.last_patch()], "Assertion %s %s" \
+        #        % (a.pending_merges(), b.last_patch())
+
+    def test_pull(self):
+        """Pull changes from one branch to another."""
+        os.mkdir('a')
+        os.chdir('a')
+
+        self.example_branch()
+        os.chdir('..')
+        self.runbzr('branch a b')
+        os.chdir('b')
+        self.runbzr('commit -m blah --unchanged')
+        os.chdir('../a')
+        a = Branch('.')
+        b = Branch('../b')
+        assert a.revision_history() == b.revision_history()[:-1]
+        self.runbzr('pull ../b')
+        assert a.revision_history() == b.revision_history()
+        self.runbzr('commit -m blah2 --unchanged')
+        os.chdir('../b')
+        self.runbzr('commit -m blah3 --unchanged')
+        self.runbzr('pull ../a', retcode=1)
+        os.chdir('../a')
+        self.runbzr('merge ../b')
+        self.runbzr('commit -m blah4 --unchanged')
+        os.chdir('../b')
+        self.runbzr('pull ../a')
+        assert a.revision_history()[-1] == b.revision_history()[-1]
+        
+
+    def test_add_reports(self):
+        """add command prints the names of added files."""
+        b = Branch('.', init=True)
+        self.build_tree(['top.txt', 'dir/', 'dir/sub.txt'])
+
+        from cStringIO import StringIO
+        out = StringIO()
+
+        ret = self.apply_redirected(None, out, None,
+                                    run_bzr,
+                                    ['add'])
+        self.assertEquals(ret, 0)
+
+        # the ordering is not defined at the moment
+        results = sorted(out.getvalue().rstrip('\n').split('\n'))
+        self.assertEquals(['added dir',
+                           'added dir/sub.txt',
+                           'added top.txt',],
+                          results)
 
 
 def has_symlinks():
@@ -233,6 +280,7 @@
     L.sort()
     return L
 
+
 class OldTests(ExternalBase):
     """old tests moved from ./testbzr."""
 

=== added file 'bzrlib/selftest/test_xml.py'
--- bzrlib/selftest/test_xml.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/test_xml.py	2005-09-06 07:26:13 +0000
@@ -0,0 +1,140 @@
+# Copyright (C) 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+from cStringIO import StringIO
+
+from bzrlib.selftest import TestCase
+from bzrlib.inventory import Inventory, InventoryEntry
+from bzrlib.xml import serializer_v4, serializer_v5
+
+_working_inventory_v4 = """<inventory file_id="TREE_ROOT">
+<entry file_id="bar-20050901064931-73b4b1138abc9cd2" kind="file" name="bar" parent_id="TREE_ROOT" />
+<entry file_id="foo-20050801201819-4139aa4a272f4250" kind="directory" name="foo" parent_id="TREE_ROOT" />
+<entry file_id="bar-20050824000535-6bc48cfad47ed134" kind="file" name="bar" parent_id="foo-20050801201819-4139aa4a272f4250" />
+</inventory>"""
+
+
+_revision_v4 = """<revision committer="Martin Pool &lt;mbp@sourcefrog.net&gt;"
+    inventory_id="mbp@sourcefrog.net-20050905080035-e0439293f8b6b9f9"
+    inventory_sha1="e79c31c1deb64c163cf660fdedd476dd579ffd41"
+    revision_id="mbp@sourcefrog.net-20050905080035-e0439293f8b6b9f9"
+    timestamp="1125907235.211783886"
+    timezone="36000">
+<message>- start splitting code for xml (de)serialization away from objects
+  preparatory to supporting multiple formats by a single library
+</message>
+<parents>
+<revision_ref revision_id="mbp@sourcefrog.net-20050905063503-43948f59fa127d92" revision_sha1="7bdf4cc8c5bdac739f8cf9b10b78cf4b68f915ff" />
+</parents>
+</revision>
+"""
+
+_revision_v5 = """<revision committer="Martin Pool &lt;mbp@sourcefrog.net&gt;"
+    inventory_id="mbp@sourcefrog.net-20050905080035-e0439293f8b6b9f9"
+    inventory_sha1="e79c31c1deb64c163cf660fdedd476dd579ffd41"
+    revision_id="mbp@sourcefrog.net-20050905080035-e0439293f8b6b9f9"
+    timestamp="1125907235.211783886"
+    timezone="36000">
+<message>- start splitting code for xml (de)serialization away from objects
+  preparatory to supporting multiple formats by a single library
+</message>
+<parents>
+<revision_ref revision_id="mbp@sourcefrog.net-20050905063503-43948f59fa127d92"/>
+</parents>
+</revision>
+"""
+
+_committed_inv_v5 = """<inventory>
+<file file_id="bar-20050901064931-73b4b1138abc9cd2" 
+      name="bar" parent_id="TREE_ROOT" 
+      text_version="mbp@foo-123123" entry_version="mbp@foo-123123"
+      />
+<directory name="subdir"
+           file_id="foo-20050801201819-4139aa4a272f4250"
+           parent_id="TREE_ROOT" 
+           entry_version="mbp@foo-00"/>
+<file file_id="bar-20050824000535-6bc48cfad47ed134" 
+      name="bar" parent_id="foo-20050801201819-4139aa4a272f4250" 
+      entry_version="mbp@foo-00"
+      text_version="mbp@foo-123123"/>
+</inventory>
+"""
+
+class TestSerializer(TestCase):
+    """Test XML serialization"""
+    def test_canned_inventory(self):
+        """Test unpacked a canned inventory v4 file."""
+        inp = StringIO(_working_inventory_v4)
+        inv = serializer_v4.read_inventory(inp)
+        self.assertEqual(len(inv), 4)
+        self.assert_('bar-20050901064931-73b4b1138abc9cd2' in inv)
+
+    def test_unpack_revision(self):
+        """Test unpacking a canned revision v4"""
+        inp = StringIO(_revision_v4)
+        rev = serializer_v4.read_revision(inp)
+        eq = self.assertEqual
+        eq(rev.committer,
+           "Martin Pool <mbp@sourcefrog.net>")
+        eq(rev.inventory_id,
+           "mbp@sourcefrog.net-20050905080035-e0439293f8b6b9f9")
+        eq(len(rev.parents), 1)
+        eq(rev.parents[0].revision_id,
+           "mbp@sourcefrog.net-20050905063503-43948f59fa127d92")
+
+    def test_unpack_revision_5(self):
+        """Test unpacking a canned revision v5"""
+        inp = StringIO(_revision_v5)
+        rev = serializer_v5.read_revision(inp)
+        eq = self.assertEqual
+        eq(rev.committer,
+           "Martin Pool <mbp@sourcefrog.net>")
+        eq(rev.inventory_id,
+           "mbp@sourcefrog.net-20050905080035-e0439293f8b6b9f9")
+        eq(len(rev.parents), 1)
+        eq(rev.timezone, 36000)
+        eq(rev.parents[0].revision_id,
+           "mbp@sourcefrog.net-20050905063503-43948f59fa127d92")
+
+    def test_unpack_inventory_5(self):
+        """Unpack canned new-style inventory"""
+        inp = StringIO(_committed_inv_v5)
+        inv = serializer_v5.read_inventory(inp)
+        eq = self.assertEqual
+        eq(len(inv), 4)
+        ie = inv['bar-20050824000535-6bc48cfad47ed134']
+        eq(ie.kind, 'file')
+        eq(ie.text_version, 'mbp@foo-123123')
+        eq(ie.entry_version, 'mbp@foo-00')
+        eq(ie.name, 'bar')
+        eq(inv[ie.parent_id].kind, 'directory')
+
+    def test_repack_inventory_5(self):
+        inp = StringIO(_committed_inv_v5)
+        inv = serializer_v5.read_inventory(inp)
+        outp = StringIO()
+        serializer_v5.write_inventory(inv, outp)
+        inv2 = serializer_v5.read_inventory(StringIO(outp.getvalue()))
+        self.assertEqual(inv, inv2)
+
+    def test_repack_revision_5(self):
+        inp = StringIO(_revision_v5)
+        rev = serializer_v5.read_revision(inp)
+        outp = StringIO()
+        serializer_v5.write_revision(rev, outp)
+        rev2 = serializer_v5.read_revision(StringIO(outp.getvalue()))
+        self.assertEqual(rev, rev2)
+

=== added file 'bzrlib/selftest/testgraph.py'
--- bzrlib/selftest/testgraph.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/testgraph.py	2005-09-10 23:46:35 +0000
@@ -0,0 +1,36 @@
+from bzrlib.selftest import TestCase
+from bzrlib.graph import farthest_nodes
+
+class TestBase(TestCase):
+    def edge_add(self, *args):
+        for start, end in zip(args[:-1], args[1:]):
+            if start not in self.graph:
+                self.graph[start] = {}
+            if end not in self.graph:
+                self.graph[end] = {}
+            self.graph[start][end] = 1
+
+    def setUp(self):
+        TestCase.setUp(self)
+        self.graph = {}
+        self.edge_add('A', 'B', 'C', 'D')
+        self.edge_add('A', 'E', 'F', 'C')
+        self.edge_add('A', 'G', 'H', 'I', 'B')
+        self.edge_add('A', 'J', 'K', 'L', 'M', 'N')
+        self.edge_add('O', 'N')
+    
+    def test_farthest(self):
+        descendants = {'A':set()}
+        for node in self.graph:
+            for ancestor in self.graph[node]:
+                if ancestor not in descendants:
+                    descendants[ancestor] = set()
+                descendants[ancestor].add(node)
+        nodes = farthest_nodes(self.graph, descendants, 'A')
+        self.assertEqual(nodes[0], 'D')
+        assert nodes[1] in ('N', 'C')
+        assert nodes[2] in ('N', 'C')
+        assert nodes[3] in ('B', 'M')
+        assert nodes[4] in ('B', 'M')
+
+

=== modified file 'bzrlib/selftest/testinv.py'
--- bzrlib/selftest/testinv.py	2005-08-25 12:46:42 +0000
+++ bzrlib/selftest/testinv.py	2005-09-05 08:00:35 +0000
@@ -49,3 +49,11 @@
         self.assertEqual(inv.path2id('src/bye.c'), 'bye-id')
         
         self.assert_('src-id' in inv)
+
+
+    def test_version(self):
+        """Inventory remembers the text's version."""
+        inv = Inventory()
+        ie = inv.add_path('foo.txt', 'file')
+        ## XXX
+

=== added file 'bzrlib/selftest/testmerge.py'
--- bzrlib/selftest/testmerge.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/testmerge.py	2005-09-05 07:10:59 +0000
@@ -0,0 +1,12 @@
+from bzrlib.branch import Branch
+from bzrlib.commit import commit
+from bzrlib.selftest import TestCaseInTempDir
+from bzrlib.merge import merge
+class TestMerge(TestCaseInTempDir):
+    """Test appending more than one revision"""
+    def test_pending(self):
+        br = Branch(".", init=True)
+        commit(br, "lala!")
+        self.assertEquals(len(br.pending_merges()), 0)
+        merge(['.', -1], [None, None])
+        self.assertEquals(len(br.pending_merges()), 0)

=== modified file 'bzrlib/selftest/testrevision.py'
--- bzrlib/selftest/testrevision.py	2005-09-07 16:31:54 +0000
+++ bzrlib/selftest/testrevision.py	2005-09-12 12:49:28 +0000
@@ -128,18 +128,119 @@
         self.assertRaises(NoSuchRevision, self.intervene, 'b@u-0-10', 
                           'c@u-0-6', self.br2.revision_history())
 
+class TestIntermediateRevisions(TestCaseInTempDir):
+
+    def setUp(self):
+        from bzrlib.commit import commit
+        TestCaseInTempDir.setUp(self)
+        self.br1, self.br2 = make_branches()
+        commit(self.br2, "Commit eleven", rev_id="b@u-0-7")
+        commit(self.br2, "Commit twelve", rev_id="b@u-0-8")
+        commit(self.br2, "Commit thirtteen", rev_id="b@u-0-9")
+        self.br1.add_pending_merge(self.br2.revision_history()[6])
+        commit(self.br1, "Commit fourtten", rev_id="a@u-0-6")
+        self.br2.add_pending_merge(self.br1.revision_history()[6])
+        commit(self.br2, "Commit fifteen", rev_id="b@u-0-10")
+
+        from bzrlib.revision import MultipleRevisionSources
+        self.sources = MultipleRevisionSources(self.br1, self.br2)
+
+    def intervene(self, ancestor, revision, revision_history=None):
+        from bzrlib.revision import get_intervening_revisions
+        return get_intervening_revisions(ancestor,revision, self.sources, 
+                                         revision_history)
+
+    def test_intervene(self):
+        """Find intermediate revisions, without requiring history"""
+        from bzrlib.errors import NotAncestor, NoSuchRevision
+        assert len(self.intervene('a@u-0-0', 'a@u-0-0')) == 0
+        self.assertEqual(self.intervene('a@u-0-0', 'a@u-0-1'), ['a@u-0-1'])
+        self.assertEqual(self.intervene('a@u-0-0', 'a@u-0-2'), 
+                         ['a@u-0-1', 'a@u-0-2'])
+        self.assertEqual(self.intervene('a@u-0-0', 'b@u-0-3'), 
+                         ['a@u-0-1', 'a@u-0-2', 'b@u-0-3'])
+        self.assertEqual(self.intervene('b@u-0-3', 'a@u-0-3'), 
+                         ['b@u-0-4', 'a@u-0-3'])
+        self.assertEqual(self.intervene('a@u-0-2', 'a@u-0-3', 
+                                        self.br1.revision_history()), 
+                         ['a@u-0-3'])
+        self.assertEqual(self.intervene('a@u-0-0', 'a@u-0-5', 
+                                        self.br1.revision_history()), 
+                         ['a@u-0-1', 'a@u-0-2', 'a@u-0-3', 'a@u-0-4', 
+                          'a@u-0-5'])
+        self.assertEqual(self.intervene('a@u-0-0', 'b@u-0-6', 
+                         self.br1.revision_history()), 
+                         ['a@u-0-1', 'a@u-0-2', 'a@u-0-3', 'a@u-0-4', 
+                          'b@u-0-6'])
+        self.assertEqual(self.intervene('a@u-0-0', 'b@u-0-5'), 
+                         ['a@u-0-1', 'a@u-0-2', 'b@u-0-3', 'b@u-0-4', 
+                          'b@u-0-5'])
+        self.assertEqual(self.intervene('b@u-0-3', 'b@u-0-6', 
+                         self.br2.revision_history()), 
+                         ['b@u-0-4', 'b@u-0-5', 'b@u-0-6'])
+        self.assertEqual(self.intervene('b@u-0-6', 'b@u-0-10'), 
+                         ['b@u-0-7', 'b@u-0-8', 'b@u-0-9', 'b@u-0-10'])
+        self.assertEqual(self.intervene('b@u-0-6', 'b@u-0-10', 
+                                        self.br2.revision_history()), 
+                         ['b@u-0-7', 'b@u-0-8', 'b@u-0-9', 'b@u-0-10'])
+        self.assertRaises(NotAncestor, self.intervene, 'b@u-0-10', 'b@u-0-6', 
+                          self.br2.revision_history())
+        self.assertRaises(NoSuchRevision, self.intervene, 'c@u-0-10', 
+                          'b@u-0-6', self.br2.revision_history())
+        self.assertRaises(NoSuchRevision, self.intervene, 'b@u-0-10', 
+                          'c@u-0-6', self.br2.revision_history())
+
 
 class TestCommonAncestor(TestCaseInTempDir):
     """Test checking whether a revision is an ancestor of another revision"""
 
+    def test_old_common_ancestor(self):
+        """Pick a resonable merge base using the old functionality"""
+        from bzrlib.revision import find_present_ancestors
+        from bzrlib.revision import old_common_ancestor as common_ancestor
+        from bzrlib.revision import MultipleRevisionSources
+        br1, br2 = make_branches()
+        revisions = br1.revision_history()
+        revisions_2 = br2.revision_history()
+        sources = MultipleRevisionSources(br1, br2)
+        expected_ancestors_list = {revisions[3]:(0, 0), 
+                                   revisions[2]:(1, 1),
+                                   revisions_2[4]:(2, 1), 
+                                   revisions[1]:(3, 2),
+                                   revisions_2[3]:(4, 2),
+                                   revisions[0]:(5, 3) }
+        ancestors_list = find_present_ancestors(revisions[3], sources)
+        assert len(expected_ancestors_list) == len(ancestors_list)
+        for key, value in expected_ancestors_list.iteritems():
+            self.assertEqual(ancestors_list[key], value, 
+                              "key %r, %r != %r" % (key, ancestors_list[key],
+                                                    value))
+        self.assertEqual(common_ancestor(revisions[0], revisions[0], sources),
+                          revisions[0])
+        self.assertEqual(common_ancestor(revisions[1], revisions[2], sources),
+                          revisions[1])
+        self.assertEqual(common_ancestor(revisions[1], revisions[1], sources),
+                          revisions[1])
+        self.assertEqual(common_ancestor(revisions[2], revisions_2[4], sources),
+                          revisions[2])
+        self.assertEqual(common_ancestor(revisions[3], revisions_2[4], sources),
+                          revisions_2[4])
+        self.assertEqual(common_ancestor(revisions[4], revisions_2[5], sources),
+                          revisions_2[4])
+        self.assertEqual(common_ancestor(revisions[5], revisions_2[6], sources),
+                          revisions[4])
+        self.assertEqual(common_ancestor(revisions_2[6], revisions[5], sources),
+                          revisions_2[5])
+
     def test_common_ancestor(self):
-        from bzrlib.revision import find_present_ancestors, common_ancestor
+        """Pick a reasonable merge base"""
+        from bzrlib.revision import find_present_ancestors
+        from bzrlib.revision import common_ancestor
         from bzrlib.revision import MultipleRevisionSources
         br1, br2 = make_branches()
         revisions = br1.revision_history()
         revisions_2 = br2.revision_history()
         sources = MultipleRevisionSources(br1, br2)
-
         expected_ancestors_list = {revisions[3]:(0, 0), 
                                    revisions[2]:(1, 1),
                                    revisions_2[4]:(2, 1), 
@@ -152,7 +253,6 @@
             self.assertEqual(ancestors_list[key], value, 
                               "key %r, %r != %r" % (key, ancestors_list[key],
                                                     value))
-
         self.assertEqual(common_ancestor(revisions[0], revisions[0], sources),
                           revisions[0])
         self.assertEqual(common_ancestor(revisions[1], revisions[2], sources),
@@ -168,7 +268,8 @@
         self.assertEqual(common_ancestor(revisions[5], revisions_2[6], sources),
                           revisions[4])
         self.assertEqual(common_ancestor(revisions_2[6], revisions[5], sources),
-                          revisions_2[5])
+                          revisions[4])
+
 
 class TestCreateSignedRevision(TestCaseInTempDir):
 
@@ -177,6 +278,7 @@
         # create a revision, sign it, apply to the store.
         pass
 
+
 class TestOperators(TestCase):
 
     def test___eq__(self):

=== modified file 'bzrlib/selftest/versioning.py'
--- bzrlib/selftest/versioning.py	2005-08-30 03:10:32 +0000
+++ bzrlib/selftest/versioning.py	2005-09-12 01:29:49 +0000
@@ -17,7 +17,6 @@
 
 """Tests of simple versioning operations"""
 
-# TODO: test adding a file whose directory is not versioned
 # TODO: test trying to commit within a directory that is not yet added
 
 
@@ -55,11 +54,11 @@
         self.assertEquals(delta.added[0][0], 'foo')
         self.failIf(delta.modified)
 
-    def test_add_in_unversioned(self):
+    def test_branch_add_in_unversioned(self):
         """Try to add a file in an unversioned directory.
 
-        smart_add may eventually add the parent as necessary, but simple
-        branch add doesn't do that.
+        "bzr add" adds the parent as necessary, but simple branch add
+        doesn't do that.
         """
         from bzrlib.branch import Branch
         from bzrlib.errors import NotVersionedError
@@ -75,7 +74,53 @@
         
         self.check_and_upgrade()
 
-        
+
+    def test_add_in_unversioned(self):
+        """Try to add a file in an unversioned directory.
+
+        "bzr add" should add the parent(s) as necessary.
+        """
+        from bzrlib.branch import Branch
+        from bzrlib.commands import run_bzr
+        eq = self.assertEqual
+
+        b = Branch('.', init=True)
+
+        self.build_tree(['inertiatic/', 'inertiatic/esp'])
+        eq(list(b.unknowns()), ['inertiatic'])
+        run_bzr(['add', 'inertiatic/esp'])
+        eq(list(b.unknowns()), [])
+
+        # Multiple unversioned parents
+        self.build_tree(['veil/', 'veil/cerpin/', 'veil/cerpin/taxt'])
+        eq(list(b.unknowns()), ['veil'])
+        run_bzr(['add', 'veil/cerpin/taxt'])
+        eq(list(b.unknowns()), [])
+
+        # Check whacky paths work
+        self.build_tree(['cicatriz/', 'cicatriz/esp'])
+        eq(list(b.unknowns()), ['cicatriz'])
+        run_bzr(['add', 'inertiatic/../cicatriz/esp'])
+        eq(list(b.unknowns()), [])
+
+    def test_add_in_versioned(self):
+        """Try to add a file in a versioned directory.
+
+        "bzr add" should do this happily.
+        """
+        from bzrlib.branch import Branch
+        from bzrlib.commands import run_bzr
+        eq = self.assertEqual
+
+        b = Branch('.', init=True)
+
+        self.build_tree(['inertiatic/', 'inertiatic/esp'])
+        eq(list(b.unknowns()), ['inertiatic'])
+        run_bzr(['add', '--no-recurse', 'inertiatic'])
+        eq(list(b.unknowns()), ['inertiatic/esp'])
+        run_bzr(['add', 'inertiatic/esp'])
+        eq(list(b.unknowns()), [])
+
     def test_subdir_add(self):
         """Add in subdirectory should add only things from there down"""
         
@@ -131,7 +176,6 @@
         
 
         
-        
 class SubdirCommit(TestCaseInTempDir):
 
     def test_subdir_commit(self):

=== modified file 'bzrlib/status.py'
--- bzrlib/status.py	2005-08-27 04:42:41 +0000
+++ bzrlib/status.py	2005-09-07 23:14:30 +0000
@@ -21,16 +21,22 @@
                 show_ids=False,
                 to_file=None,
                 show_pending=True):
-    """Display single-line status for non-ignored working files.
+    """Display status for non-ignored working files.
 
-    show_all
-        If true, show unmodified files too.
+    show_unchanged
+        If set, includes unchanged files.
 
     specific_files
         If set, only show the status of files in this list.
 
+    show_ids
+        If set, includes each file's id.
+
     to_file
         If set, write to this file (default stdout.)
+
+    show_pending
+        If set, write pending merges.
     """
     import sys
     from bzrlib.delta import compare_trees

=== modified file 'bzrlib/store.py'
--- bzrlib/store.py	2005-09-06 16:06:41 +0000
+++ bzrlib/store.py	2005-09-12 12:28:19 +0000
@@ -91,10 +91,12 @@
         super(ImmutableStore, self).__init__()
         self._basedir = basedir
 
-    def _path(self, id):
-        if '\\' in id or '/' in id:
-            raise ValueError("invalid store id %r" % id)
-        return os.path.join(self._basedir, id)
+    def _path(self, entry_id):
+        if not isinstance(entry_id, basestring):
+            raise TypeError(type(entry_id))
+        if '\\' in entry_id or '/' in entry_id:
+            raise ValueError("invalid store id %r" % entry_id)
+        return os.path.join(self._basedir, entry_id)
 
     def __repr__(self):
         return "%s(%r)" % (self.__class__.__name__, self._basedir)

=== modified file 'bzrlib/upgrade.py'
--- bzrlib/upgrade.py	2005-08-29 10:57:01 +0000
+++ bzrlib/upgrade.py	2005-09-05 09:27:11 +0000
@@ -15,8 +15,11 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+import tempfile, os, errno
+                
 import bzrlib.errors
 import bzrlib.progress
+from bzrlib.xml import serializer_v4
 
 
 def upgrade(branch):
@@ -96,11 +99,8 @@
                 # revision entry. I'm not supremely happy about it, but
                 # there should be *some* way of making old entries have
                 # the full meta information.
-                import tempfile, os, errno
-                from bzrlib.xml import pack_xml
-                
                 rev_tmp = tempfile.TemporaryFile()
-                pack_xml(rev, rev_tmp)
+                serializer_v4.write_revision(rev, rev_tmp)
                 rev_tmp.seek(0)
 
                 tmpfd, tmp_path = tempfile.mkstemp(prefix=rev_id, suffix='.gz',

=== modified file 'bzrlib/xml.py'
--- bzrlib/xml.py	2005-08-25 05:58:05 +0000
+++ bzrlib/xml.py	2005-09-12 12:28:19 +0000
@@ -20,21 +20,345 @@
 # "XML is like violence: if it doesn't solve your problem, you aren't
 # using enough of it." -- various
 
-# importing this module is fairly slow because it has to load several ElementTree bits
+# importing this module is fairly slow because it has to load several
+# ElementTree bits
+
 try:
     from util.cElementTree import ElementTree, SubElement, Element
 except ImportError:
     from util.elementtree.ElementTree import ElementTree, SubElement, Element
 
-
-def pack_xml(o, f):
-    """Write object o to file f as XML.
-
-    o must provide a to_element method.
-    """
-    ElementTree(o.to_element()).write(f, 'utf-8')
-    f.write('\n')
-
-
-def unpack_xml(cls, f):
-    return cls.from_element(ElementTree().parse(f))
+from bzrlib.inventory import ROOT_ID, Inventory, InventoryEntry
+from bzrlib.revision import Revision, RevisionReference        
+from bzrlib.errors import BzrError
+
+
+class Serializer(object):
+    """Abstract object serialize/deserialize"""
+    def write_inventory(self, inv, f):
+        """Write inventory to a file"""
+        elt = self._pack_inventory(inv)
+        self._write_element(elt, f)
+
+    def read_inventory(self, f):
+        return self._unpack_inventory(self._read_element(f))
+
+    def write_revision(self, rev, f):
+        self._write_element(self._pack_revision(rev), f)
+
+    def read_revision(self, f):
+        return self._unpack_revision(self._read_element(f))
+
+    def _write_element(self, elt, f):
+        ElementTree(elt).write(f, 'utf-8')
+        f.write('\n')
+
+    def _read_element(self, f):
+        return ElementTree().parse(f)
+
+
+
+class _Serializer_v4(Serializer):
+    """Version 0.0.4 serializer
+
+    You should use the serialzer_v4 singleton."""
+    
+    __slots__ = []
+    
+    def _pack_inventory(self, inv):
+        """Convert to XML Element"""
+        e = Element('inventory')
+        e.text = '\n'
+        if inv.root.file_id not in (None, ROOT_ID):
+            e.set('file_id', inv.root.file_id)
+        for path, ie in inv.iter_entries():
+            e.append(self._pack_entry(ie))
+        return e
+
+
+    def _pack_entry(self, ie):
+        """Convert InventoryEntry to XML element"""
+        e = Element('entry')
+        e.set('name', ie.name)
+        e.set('file_id', ie.file_id)
+        e.set('kind', ie.kind)
+
+        if ie.text_size != None:
+            e.set('text_size', '%d' % ie.text_size)
+
+        for f in ['text_id', 'text_sha1', 'symlink_target']:
+            v = getattr(ie, f)
+            if v != None:
+                e.set(f, v)
+
+        # to be conservative, we don't externalize the root pointers
+        # for now, leaving them as null in the xml form.  in a future
+        # version it will be implied by nested elements.
+        if ie.parent_id != ROOT_ID:
+            assert isinstance(ie.parent_id, basestring)
+            e.set('parent_id', ie.parent_id)
+
+        e.tail = '\n'
+
+        return e
+
+
+    def _unpack_inventory(self, elt):
+        """Construct from XML Element
+        """
+        assert elt.tag == 'inventory'
+        root_id = elt.get('file_id') or ROOT_ID
+        inv = Inventory(root_id)
+        for e in elt:
+            ie = self._unpack_entry(e)
+            if ie.parent_id == ROOT_ID:
+                ie.parent_id = root_id
+            inv.add(ie)
+        return inv
+
+
+    def _unpack_entry(self, elt):
+        assert elt.tag == 'entry'
+
+        ## original format inventories don't have a parent_id for
+        ## nodes in the root directory, but it's cleaner to use one
+        ## internally.
+        parent_id = elt.get('parent_id')
+        if parent_id == None:
+            parent_id = ROOT_ID
+
+        ie = InventoryEntry(elt.get('file_id'),
+                              elt.get('name'),
+                              elt.get('kind'),
+                              parent_id)
+        ie.text_id = elt.get('text_id')
+        ie.text_sha1 = elt.get('text_sha1')
+        ie.symlink_target = elt.get('symlink_target')
+
+        ## mutter("read inventoryentry: %r" % (elt.attrib))
+
+        v = elt.get('text_size')
+        ie.text_size = v and int(v)
+
+        return ie
+
+
+    def _pack_revision(self, rev):
+        """Revision object -> xml tree"""
+        root = Element('revision',
+                       committer = rev.committer,
+                       timestamp = '%.9f' % rev.timestamp,
+                       revision_id = rev.revision_id,
+                       inventory_id = rev.inventory_id,
+                       inventory_sha1 = rev.inventory_sha1,
+                       )
+        if rev.timezone:
+            root.set('timezone', str(rev.timezone))
+        root.text = '\n'
+
+        msg = SubElement(root, 'message')
+        msg.text = rev.message
+        msg.tail = '\n'
+
+        if rev.parents:
+            pelts = SubElement(root, 'parents')
+            pelts.tail = pelts.text = '\n'
+            for rr in rev.parents:
+                assert isinstance(rr, RevisionReference)
+                p = SubElement(pelts, 'revision_ref')
+                p.tail = '\n'
+                assert rr.revision_id
+                p.set('revision_id', rr.revision_id)
+                if rr.revision_sha1:
+                    p.set('revision_sha1', rr.revision_sha1)
+
+        return root
+
+    
+    def _unpack_revision(self, elt):
+        """XML Element -> Revision object"""
+        
+        # <changeset> is deprecated...
+        if elt.tag not in ('revision', 'changeset'):
+            raise BzrError("unexpected tag in revision file: %r" % elt)
+
+        rev = Revision(committer = elt.get('committer'),
+                       timestamp = float(elt.get('timestamp')),
+                       revision_id = elt.get('revision_id'),
+                       inventory_id = elt.get('inventory_id'),
+                       inventory_sha1 = elt.get('inventory_sha1')
+                       )
+
+        precursor = elt.get('precursor')
+        precursor_sha1 = elt.get('precursor_sha1')
+
+        pelts = elt.find('parents')
+
+        if pelts:
+            for p in pelts:
+                assert p.tag == 'revision_ref', \
+                       "bad parent node tag %r" % p.tag
+                rev_ref = RevisionReference(p.get('revision_id'),
+                                            p.get('revision_sha1'))
+                rev.parents.append(rev_ref)
+
+            if precursor:
+                # must be consistent
+                prec_parent = rev.parents[0].revision_id
+                assert prec_parent == precursor
+        elif precursor:
+            # revisions written prior to 0.0.5 have a single precursor
+            # give as an attribute
+            rev_ref = RevisionReference(precursor, precursor_sha1)
+            rev.parents.append(rev_ref)
+
+        v = elt.get('timezone')
+        rev.timezone = v and int(v)
+
+        rev.message = elt.findtext('message') # text of <message>
+        return rev
+
+
+
+class _Serializer_v5(Serializer):
+    """Version 5 serializer
+
+    Packs objects into XML and vice versa.
+
+    You should use the serialzer_v5 singleton."""
+    
+    __slots__ = []
+    
+    def _pack_inventory(self, inv):
+        """Convert to XML Element"""
+        e = Element('inventory')
+        e.text = '\n'
+        if inv.root.file_id not in (None, ROOT_ID):
+            e.set('file_id', inv.root.file_id)
+        for path, ie in inv.iter_entries():
+            e.append(self._pack_entry(ie))
+        return e
+
+
+    def _pack_entry(self, ie):
+        """Convert InventoryEntry to XML element"""
+        assert ie.kind == 'directory' or ie.kind == 'file'
+        e = Element(ie.kind)
+        e.set('name', ie.name)
+        e.set('file_id', ie.file_id)
+
+        if ie.text_size != None:
+            e.set('text_size', '%d' % ie.text_size)
+
+        for f in ['text_version', 'text_sha1', 'entry_version']:
+            v = getattr(ie, f)
+            if v != None:
+                e.set(f, v)
+
+        # to be conservative, we don't externalize the root pointers
+        # for now, leaving them as null in the xml form.  in a future
+        # version it will be implied by nested elements.
+        if ie.parent_id != ROOT_ID:
+            assert isinstance(ie.parent_id, basestring)
+            e.set('parent_id', ie.parent_id)
+
+        e.tail = '\n'
+
+        return e
+
+
+    def _pack_revision(self, rev):
+        """Revision object -> xml tree"""
+        root = Element('revision',
+                       committer = rev.committer,
+                       timestamp = '%.9f' % rev.timestamp,
+                       revision_id = rev.revision_id,
+                       inventory_id = rev.inventory_id,
+                       inventory_sha1 = rev.inventory_sha1,
+                       )
+        if rev.timezone:
+            root.set('timezone', str(rev.timezone))
+        root.text = '\n'
+
+        msg = SubElement(root, 'message')
+        msg.text = rev.message
+        msg.tail = '\n'
+
+        if rev.parents:
+            pelts = SubElement(root, 'parents')
+            pelts.tail = pelts.text = '\n'
+            for rr in rev.parents:
+                assert isinstance(rr, RevisionReference)
+                p = SubElement(pelts, 'revision_ref')
+                p.tail = '\n'
+                assert rr.revision_id
+                p.set('revision_id', rr.revision_id)
+
+        return root
+
+    
+
+    def _unpack_inventory(self, elt):
+        """Construct from XML Element
+        """
+        assert elt.tag == 'inventory'
+        root_id = elt.get('file_id') or ROOT_ID
+        inv = Inventory(root_id)
+        for e in elt:
+            ie = self._unpack_entry(e)
+            if ie.parent_id == ROOT_ID:
+                ie.parent_id = root_id
+            inv.add(ie)
+        return inv
+
+
+    def _unpack_entry(self, elt):
+        kind = elt.tag
+        assert kind == 'directory' or kind == 'file'
+
+        parent_id = elt.get('parent_id')
+        if parent_id == None:
+            parent_id = ROOT_ID
+
+        ie = InventoryEntry(elt.get('file_id'),
+                            elt.get('name'),
+                            kind,
+                            parent_id)
+        ie.text_version = elt.get('text_version')
+        ie.entry_version = elt.get('entry_version')
+        ie.text_sha1 = elt.get('text_sha1')
+        v = elt.get('text_size')
+        ie.text_size = v and int(v)
+
+        return ie
+
+
+    def _unpack_revision(self, elt):
+        """XML Element -> Revision object"""
+        assert elt.tag == 'revision'
+        
+        rev = Revision(committer = elt.get('committer'),
+                       timestamp = float(elt.get('timestamp')),
+                       revision_id = elt.get('revision_id'),
+                       inventory_id = elt.get('inventory_id'),
+                       inventory_sha1 = elt.get('inventory_sha1')
+                       )
+
+        for p in elt.find('parents'):
+            assert p.tag == 'revision_ref', \
+                   "bad parent node tag %r" % p.tag
+            rev_ref = RevisionReference(p.get('revision_id'))
+            rev.parents.append(rev_ref)
+
+        v = elt.get('timezone')
+        rev.timezone = v and int(v)
+
+        rev.message = elt.findtext('message') # text of <message>
+        return rev
+
+
+
+"""singleton instance"""
+serializer_v4 = _Serializer_v4()
+
+serializer_v5 = _Serializer_v5()

=== modified file 'testsweet.py'
--- testsweet.py	2005-09-06 11:09:03 +0000
+++ testsweet.py	2005-09-09 13:44:00 +0000
@@ -165,9 +165,12 @@
     runner = TextTestRunner(stream=sys.stdout,
                             descriptions=0,
                             verbosity=verbosity)
-    visitor = filteringVisitor(pattern)
-    suite.visit(visitor)
-    result = runner.run(visitor.suite())
+    if not pattern or pattern == ".*":
+        result = runner.run(suite)
+    else:
+        visitor = filteringVisitor(pattern)
+        suite.visit(visitor)
+        result = runner.run(visitor.suite())
     # This is still a little bogus, 
     # but only a little. Folk not using our testrunner will
     # have to delete their temp directories themselves.

